﻿using PComm;
using PObject;
using PUtility;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ClientForm
{
	public partial class MainForm : Form
	{
		private PClient heartBeatClient;
		private PClient client;

		private System.Timers.Timer heartBeatTimer;
		private System.Timers.Timer testTimer;

		public MainForm()
		{
			InitializeComponent();
			InitInstance();
		}

		private void InitInstance()
		{
			string clientId = PUtil.GetRandomString(5);
			string heartBeatClientId = clientId + "_hb";

			heartBeatClient = new PClient("127.0.0.1", 7778, heartBeatClientId);
			heartBeatClient.OnReceived += HeartBeatClient_OnReceived;

			client = new PClient("127.0.0.1", 7777, clientId);
			client.OnReceived += Client_OnReceived;

			heartBeatTimer = new System.Timers.Timer();
			heartBeatTimer.Interval = 5000;
			heartBeatTimer.Elapsed += HeartBeatTimer_Elapsed;
			heartBeatTimer.Start();

			testTimer = new System.Timers.Timer();
			testTimer.Interval = 500;
			testTimer.Elapsed += TestTimer_Elapsed;
			testTimer.Start();
		}

		private void WriteLog(string log)
		{
			if (this.InvokeRequired)
			{
				Invoke((MethodInvoker)delegate () { WriteLog(log); });
			}
			else
			{
				logConsole.AppendText($"[{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}] {log}" + Environment.NewLine);
				//PUtil.ScrollToEnd(logConsole);
			}
		}

		private void HeartBeatTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			heartBeatClient.Send(PServer.HB_CHECK);
		}

		private void HeartBeatClient_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			if (dataType != PDataType.SimpleString)
			{
				WriteLog("ERROR!!! WRONG DATA TYPE FOR HEART BEAT!!!");
				return;
			}

			string msg = Encoding.UTF8.GetString(data, 0, data.Length);
			if (msg != PServer.HB_CHECK)
				WriteLog("WARNING!!! BAD HEARTBEAT!!!");
		}

		private void Client_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			throw new NotImplementedException();
		}

		private void TestTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			//button1_Click(sender, e);
		}

		private void stringButton_Click(object sender, EventArgs e)
		{
			client.Send("String send test!");
		}

		private void objectButton_Click(object sender, EventArgs e)
		{
			Random rand = new Random();
			Person person = new Person()
			{
				Name = PUtil.GetRandomString(5),
				Age = rand.Next(20, 100)
			};

			byte[] data = PSerializer.Serialize(person);
			client.Send(PDataType.Person, data);
		}

		private void imageButton_Click(object sender, EventArgs e)
		{
			OpenFileDialog dlg = new OpenFileDialog();
			dlg.Filter = "JPG File|*.jpg|PNG File|*.png|All File|*.*";
			if (dlg.ShowDialog() == DialogResult.Cancel)
				return;

			Image img = Image.FromFile(dlg.FileName);
			byte[] data = PUtil.ImageToByteArray(img);
			client.Send(PDataType.Image, data);

			GC.Collect();
		}
	}
}
